function ar_shift = dar(absorption_ratios, fraction)

%#
%# # License
%#
%# =============================================================================
%#
%# Copyright 2011, Dimitrios Bisias, Andrew W. Lo, and Stavros Valavanis
%#
%# COPYRIGHT STATUS:  This work was funded in whole or in part by the Office of 
%# Financial Research under U.S. Government contract TOSOFR-11-C-0001, and is, 
%# therefore, subject to the following license: The Government is granted for 
%# itself and others acting on its behalf a paid-up, nonexclusive, irrevocable, 
%# worldwide license to reproduce, prepare derivative works,
%# distribute copies to the public, perform and display the work.  
%# All other rights are reserved by the copyright owner.
%# 
%# THIS SOFTWARE IS PROVIDED "AS IS".  YOU ARE USING THIS SOFTWARE AT YOUR OWN
%# RISK.  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
%# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
%# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS, CONTRIBUTORS, OR
%# THE UNITED STATES GOVERNMENT BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
%# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
%# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
%# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
%# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
%# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
%# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%# 
%# =============================================================================
%#
%# # Synopsis
%#	
%# ar_shift - Calculates the standardized AR shift from AR based on Kritzman 
%# et al. (2010)
%#		
%# # Introduction 
%# NOTE: PART OF A SET OF 2 RELATED FILES: 
%#
%#  * dar.m
%#  * absorption_ratio.m
%#
%# Kritzman, Li, Page, and Rigobon (2010) propose to measure systemic risk via 
%# the Absorption Ratio (AR), which they define as the fraction of the total 
%# variance of a set of asset returns explained or absorbed by a fixed 
%# number of eigenvectors. The absorption ratio captures the extent to which 
%# markets are unified or tightly coupled. When markets are tightly coupled,
%# they become more fragile in the sense that negative shocks propagate more 
%# quickly and broadly than when markets are loosely linked. The authors apply 
%# their AR analysis to several broad markets, introduce a standardized measure
%# of shifts in the AR, and analyze how these shifts relate to changes in asset
%# prices and financial turbulence.
%#
%# The authors also propose a technical indicator of AR movements which they 
%# find to be a leading indicator of trouble for asset prices.  This is known
%# as the standardized AR shift.
%#
%# # Input Description
%#
%# ## Input arg: absorption_ratios
%# The fraction of the total variance of a set of asset returns explained 
%# or absorbed by a fixed number of eigenvectors.  In the paper absorption
%# ratios are given for 1 year.
%# ### Type: float
%# ### Range: (0,1]
%# ### Dimensions: Tx1 matrix
%#
%# 1.  Rows represent each of T periods at which rolling AR was calculated. 
%#
%#
%# ## Input arg: fraction
%# Number of days in the short term absorption ratio over number of days in 
%# the long term absorption ratio. In the paper 15/252 if we consider a year 
%# having 252 absorption ratios.
%# ### Type: float
%# ### Range: (0,1)
%# ### Dimensions: scalar
%#
%# # Output Description
%#
%# ## Output arg: ar_shift
%# Number of days used to create short term moving average of absorption 
%# ratio over number of days used in creating a long term moving average. 
%# The paper uses 15/252 assuming a year has 252 absorption ratios.
%# ### Type: float
%# ### Range: (-inf,+inf)
%# ### Dimensions: scalar
%#
%# # Examples
%# NOTE: Numbers used in the examples are arbitrary valid values.  
%# They do not necessarily represent a realistic or plausible scenario.
%#
%#     absorption_ratios = [.23, .25, .11, .50, .9]';
%#
%#     fraction = .2;
%#
%#     ar_shift = dar(absorption_ratios, fraction);
%#
%# # References
%#
%# [Kritzman, et al. (2010). Principal components as a measure of systemic 
%# risk. SSRN eLibrary.] 
%# (http://papers.ssrn.com/sol3/papers.cfm?abstract_id=1582687)
%#
%# [Bisias et al. (2012). A survey of systemic risk analytics 
%# (Working paper #0001). Washington, DC: Office of Financial Research, 98-100.]
%# (http://www.treasury.gov/initiatives/wsr/ofr/Documents/
%# OFRwp0001_BisiasFloodLoValavanis_ASurveyOfSystemicRiskAnalytics.pdf)
%#
%# # Code

% Run warning message
warning('OFRwp0001:UntestedCode', ...
    ['This version of the source code is very preliminary, ' ...
     'and has not been thoroughly tested. Users should not rely on ' ...
     'these calculations.']);





   [num_rows, num_cols] = size(absorption_ratios);
   if (num_rows ~= 1) && (num_cols ~=1)
    error('The absorption_ratios is a 1-d vector');
   end
   
   num_days = length(absorption_ratios);
   num_days_short_term = round(fraction*num_days);
   
   % Calculate the AR shift
   % For the short term absorption ratio take the last num_days_short_term days
   numerator = mean(absorption_ratios(end-num_days_short_term+1:end)) ...
   - mean(absorption_ratios);
   denominator = std(absorption_ratios);
   ar_shift = numerator/denominator;
   