function directed = directed_banking_linkages(initial_net_bank_claims, ... 
initial_net_non_bank_claims, final_net_bank_claims, final_net_non_bank_claims)
%
%#
%# # License
%#
%# =============================================================================
%#
%# Copyright 2011, Dimitrios Bisias, Andrew W. Lo, and Stavros Valavanis
%#
%# COPYRIGHT STATUS:  This work was funded in whole or in part by the Office of 
%# Financial Research under U.S. Government contract TOSOFR-11-C-0001, and is, 
%# therefore, subject to the following license: The Government is granted for 
%# itself and others acting on its behalf a paid-up, nonexclusive, irrevocable, 
%# worldwide license to reproduce, prepare derivative works,
%# distribute copies to the public, perform and display the work.  
%# All other rights are reserved by the copyright owner.
%# 
%# THIS SOFTWARE IS PROVIDED "AS IS".  YOU ARE USING THIS SOFTWARE AT YOUR OWN
%# RISK.  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
%# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
%# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS, CONTRIBUTORS, OR
%# THE UNITED STATES GOVERNMENT BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
%# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
%# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
%# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
%# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
%# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
%# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%# 
%# =============================================================================
%#
%# # Synopsis
%#			
%# directed_banking_linkages - Calculates the directed network of banking 
%# linkages for n countries between two time instants t1 and t2 based on 
%# Fender and McGuire (2010)
%# 
%# # Introduction
%#
%# Fender and McGuire (2010) emphasize the fact that large, international 
%# banks tend to have offices in many different countries, hence focusing on 
%# banking-group-level balance-sheet risks may ignore important exposures at 
%# the individual-office level since these risks are netted out at the group 
%# level. Such individual-office frictions may be important, especially during 
%# a crisis. A key implication of geographical diversity is that cross-border 
%# linkages of individual office locations can determine how shocks are 
%# transmitted from one location (country) to another. The authors propose 
%# creating undirected and directed networks at the local-office level to 
%# assess these risks.  
%# 
%# In creating a directed network, times t1 and t2 must be defined. The 
%# thickness of an arrow is then proportional to the amount of net bank flows 
%# between regions from t1 to t2. More specifically, an arrow points from A to B 
%# if net flows in this direction are positive, calculated as changes in net 
%# interbank claims (assets minus liabilities) of banks in A on banks in B, plus
%# net claims of banks in A on non-banks in B, minus net claims of banks in B on
%# non-banks in A.
%#
%# # Input Description
%#
%# ## Input arg: initial_net_bank_claims
%# The (i,j) component shows the net bank claims from country i to banks in 
%# country j at t1.
%# ### Type: float
%# ### Range: (-inf,+inf)
%# ### Dimensions: NxN matrix (antisymmetric: A == -A')
%#
%# 1.  Rows represent each of N countries.
%# 2.  Columns represent each of N countries.
%#
%#
%# ## Input arg: initial_net_non_bank_claims
%# The (i,j) component shows the net claims from banks in country i to non-banks 
%# in country j at t1.
%# ### Type: float
%# ### Range: (-inf,+inf)
%# ### Dimensions: NxN matrix (antisymmetric: A == -A')
%#
%# 1.  Rows represent each of N countries.
%# 2.  Columns represent each of N countries.
%#
%#
%# ## Input arg: final_net_bank_claims
%# The (i,j) component shows the net bank claims from country i to banks in 
%# country j at t2.
%# ### Type: float
%# ### Range: (-inf,+inf)
%# ### Dimensions: NxN matrix (antisymmetric: A == -A')
%#
%# 1.  Rows represent each of N countries.
%# 2.  Columns represent each of N countries.
%#
%#
%# ## Input arg: final_net_non_bank_claims
%# The (i,j) component shows the net claims from banks in country i to non-banks 
%# in country j at t2.
%# ### Type: float
%# ### Range: (-inf,+inf)
%# ### Dimensions: NxN matrix (antisymmetric: A == -A')
%#
%# 1.  Rows represent each of N countries.
%# 2.  Columns represent each of N countries.
%#
%#
%# # Output Description
%#
%# ## Output arg: directed
%# A symmetric matrix with null diagonals depicting gross linkages.
%# directed network of banking linkages for N countries between two time 
%# instants t1 and t2.
%# Note: values are yes/no flags, represented as 1/0 values, respectively.
%# ### Type: float
%# ### Range: {0, 1}
%# ### Dimensions: NxN matrix, No requirements on symmetry.
%#
%# 1.  Rows represent each of N countries.
%# 2.  Columns represent each of N countries.
%#
%#
%# # Examples
%# NOTE: Numbers used in the examples are arbitrary valid values.  
%# They do not necessarily represent a realistic or plausible scenario.
%# 
%# Very simple net claims graphs:
%#
%# Bank claims:
%# 
%#     t1:               t2:  
%#     C1 -20-> C2       C1 <-5- C2
%#      \      |          \       |
%#      30    10           15     5
%#        \   \|/            \   \|/
%#         `-> C3             `-> C3
%#
%# Non-bank claims:
%#
%#     t1:               t2:  
%#     N1 <-3- N2        N1 <-5- N2
%#       \     /|\         \     /|\
%#        13    20         8     40
%#         \    |           \    |
%#          `-> N3           `-> N3
%#
%#     initial_net_bank_claims = ...
%#     [ 0,  20, 30; 
%#      -20, 0, 10; 
%#      -30,-10, 0];
%# 
%#     initial_net_non_bank_claims = ...
%#     [0, -3,  13; 
%#      3,  0, -20; 
%#     -13, 20,  0];
%# 
%#     final_net_bank_claims = ...
%#     [0, -5, 15; 
%#      5,  0, 5; 
%#     -15, -5, 0];
%# 
%#     final_net_non_bank_claims =  ...
%#     [0, -5,  8; 
%#      5,  0, -40; 
%#     -8, 40,  0];
%#
%#     directed = directed_banking_linkages(initial_net_bank_claims, ... 
%#     initial_net_non_bank_claims, final_net_bank_claims, ...
%#     final_net_non_bank_claims);
%# 
%# # References
%#
%# [Fender, I., & McGuire, P. (2010). Bank structure, funding risk and the 
%# transmission of shocks across countries: concepts and measurement. 
%# BIS quarterly review, 2010, 63-79.] 
%# (http://www.bis.org/publ/qtrpdf/r_qt1009h.htm)
%#
%# [Bisias et al. (2012). A survey of systemic risk analytics 
%# (Working paper #0001). Washington, DC: Office of Financial Research, 69-73.]
%# (http://www.treasury.gov/initiatives/wsr/ofr/Documents/
%# OFRwp0001_BisiasFloodLoValavanis_ASurveyOfSystemicRiskAnalytics.pdf)
%#
%# # Code

% Run warning message
warning('OFRwp0001:UntestedCode', ...
    ['This version of the source code is very preliminary, ' ...
     'and has not been thoroughly tested. Users should not rely on ' ...
     'these calculations.']);


initial_net_claims =  initial_net_bank_claims+initial_net_non_bank_claims ...
 - initial_net_non_bank_claims';
final_net_claims = final_net_bank_claims + final_net_non_bank_claims ...
- final_net_non_bank_claims';

% Change between t1 and t2
diff_mat = final_net_claims - initial_net_claims;

% If (i,j) element is positive put 1 otherwise 0
directed = diff_mat>0;

num_countries = size(initial_net_bank_claims,1);
% If initial_net_bank_claims and final_net_bank_claims have 0 diagonals
% this is not needed but in any case
for i=1:num_countries
    directed(i,i)=0;
end





