function coeffs = probability_liquidation_model(aums, returns, flows, ...
is_liquidated_series)
% 
%#
%# # License
%#
%# =============================================================================
%#
%# Copyright 2011, Dimitrios Bisias, Andrew W. Lo, and Stavros Valavanis
%#
%# COPYRIGHT STATUS:  This work was funded in whole or in part by the Office of 
%# Financial Research under U.S. Government contract TOSOFR-11-C-0001, and is, 
%# therefore, subject to the following license: The Government is granted for 
%# itself and others acting on its behalf a paid-up, nonexclusive, irrevocable, 
%# worldwide license to reproduce, prepare derivative works,
%# distribute copies to the public, perform and display the work.  
%# All other rights are reserved by the copyright owner.
%# 
%# THIS SOFTWARE IS PROVIDED "AS IS".  YOU ARE USING THIS SOFTWARE AT YOUR OWN
%# RISK.  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
%# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
%# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS, CONTRIBUTORS, OR
%# THE UNITED STATES GOVERNMENT BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
%# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
%# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
%# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
%# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
%# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
%# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%# 
%# =============================================================================
%#
%# # Synopsis
%#
%# probability_liquidation_model - calculates the coefficients in the logit 
%# model for fund liquidation probability. Generates coefficients as outputs
%# to parameterize the probability_liquidation function.
%#
%# # Introduction 
%# NOTE: PART OF A SET OF 2 RELATED FILES:
%#
%#  * probability_liquidation.m
%#  * probability_liquidation_model.m
%#
%# Chan, Getmansky, Haas, and Lo (2006a, 2006b) consider the broader impact 
%# of hedge funds on systemic risk by examining the unique risk/return 
%# profiles of hedge funds at both the individual-fund and 
%# aggregate-industry levels and propose three new risk measures for hedge 
%# fund investments. In the third risk measure, the authors create a measure of 
%# the probability of hedge fund liquidation by running a logit model on a set 
%# of factors driving hedge fund performance. 
%# 
%# The authors describe three metrics in their paper; we concetrate on Hedge
%# Fund Liquidation Probability. This metric is discussed extensively in 
%# Section F.7.1.2, and specifically equations (A.106) and (A.107), in 
%# Bisias et al. (2012).  Please refer to those pages for an extensive 
%# description of the code in this module and the required inputs. 
%#
%# # Input Description
%# 
%# ## Input arg: aums
%# Time series of Assets Under Management (AUM) values for different funds.
%# ### Type: float
%# ### Range: (0,+inf)
%# ### Dimensions: TxK matrix
%#
%# 1.  Rows represent dates (ascending).
%# 2.  Columns represent AUM values for each of K funds.
%#
%#
%# ## Input arg: returns
%# Time series of returns for different funds.
%# ### Type: float
%# ### Range: (-inf,+inf)
%# ### Dimensions: TxK matrix
%#
%# 1.  Rows represent dates (ascending).
%# 2.  Columns represent returns for each of K funds.
%#
%#
%# ## Input arg: flows
%# The series of flows for different funds.
%# ### Type: float
%# ### Range: (-inf,+inf)
%# ### Dimensions: TxK matrix
%#
%# 1.  Rows represent dates (ascending).
%# 2.  Columns represent flows for each of K funds.
%#
%#
%# ## Input arg: is_liquidated_series
%# Time series indicating when a hedge fund liquidation event occurs. A binary
%# matrix: 0 = no change, 1 = liquidation event. Thus (i,j) = 1 implies that
%# fund j was liquidated at time i.
%# Note: values are yes/no flags, represented as 1/0 values, respectively.
%# ### Type: float
%# ### Range: {0, 1}
%# ### Dimensions: TxK matrix
%#
%# 1.  Rows represent dates (ascending).
%# 2.  Columns represent liquidation event for each of K funds.
%#
%#
%# # Output Description
%#
%# ## Output arg: coeffs
%# Regression coefficients B0-B8 are generated as a vector ordered as follows: 
%# B0 - Age; B1 - assets at time t-1; B2, B3, B4 - returns at time t, t-1, and 
%# t-2; B5, B6, B7 - flows at time t, t-1, and t-2; B8 - a y-intercept constant. 
%# ### Type: float
%# ### Range: (-inf,+inf)
%# ### Dimensions: 9x1 matrix
%#
%# 1.  Rows represent different independent variables.
%#
%#
%# # Examples
%# NOTE: Numbers used in the examples are arbitrary valid values.  
%# They do not necessarily represent a realistic or plausible scenario.
%# 
%#     returns = ...
%#     [.05, 0.1, -0.08; ...
%#     0.1,  0.08,-0.02; ...
%#     0.09, 0.07, 0.09; ...
%#     0.1,  0.01,-0.1; ...
%#     0.06, 0.01, 0.03]
%#
%#     aums = ...
%#     [4.5, 1.7, 3.9; ...
%#     4.3, 3.9, 5.2; ...
%#     8.2, 8.3, 4.1; ...
%#     8.3, 8.0, 6.5; ...
%#     1.3, 6.0, 6.2]
%# 
%#     flows = ...
%#     [-4.6,-1.3, 1.4; ...
%#     0.6,-0.3,-1.2; ...
%#     3.8, 4.8,-3.0; ...
%#     1.6,-3.4,-0.7; ...
%#     -3.0, 3.5,-0.1]
%# 
%#     is_liquidated_series = ...
%#     [0,0,0; ...
%#     0,0,0; ...
%#     0,1,0; ...
%#     0,0,1; ...
%#     1,0,0]
%# 
%#     coeffs = probability_liquidation_model(aums, returns, flows, ...
%#     is_liquidated_series)
%# 
%# # References
%#
%# [Chan et al. (2006a). Do hedge funds increase systemic risk?. 
%# Economic Review-Federal Reserve Bank of Atlanta, 91(4), 49.] 
%# (http://web.mit.edu/~alo/www/Papers/EcRev2007.pdf)
%#
%# [Chan et al. (2006b). Systemic risk and hedge funds. The Risks of Financial 
%# Institutions. Chicago, IL: University of Chicago Press, 235-330.] 
%# (http://www.nber.org/papers/w11200.pdf) 
%#
%# [Bisias et al. (2012). A survey of systemic risk analytics 
%# (Working paper #0001). 
%# Washington, DC: Office of Financial Research, 133-135.]
%# (http://www.treasury.gov/initiatives/wsr/ofr/Documents/
%# OFRwp0001_BisiasFloodLoValavanis_ASurveyOfSystemicRiskAnalytics.pdf)
%#
%# # Code

% Run warning message
warning('OFRwp0001:UntestedCode', ...
    ['This version of the source code is very preliminary, ' ...
     'and has not been thoroughly tested. Users should not rely on ' ...
     'these calculations.']);



num_months = size(aums,1);
num_funds = size(aums,2);

% Lets form the matrix X and y for the logistic regression
% 8 is the number of regressors in the model

% Every num_months-2 group refers to a fund
X = [];
y = [];
for i=1:num_funds
    fund_matrix = [[3:num_months]' aums(2:num_months-1,i) returns( ...
	3:num_months,i) returns(2:num_months-1,i) returns(1:num_months-2,i) ...
	flows(3:num_months,i) flows(2:num_months-1,i) flows(1:num_months-2,i)];
    X = [X;fund_matrix];
    y = [y;is_liquidated_series(3:num_months,i)];
end

% It adds automatically the constant
coeffs = glmfit(X, [y ones(length(y),1)], 'binomial', 'link', 'logit');