function [thetas, ksi] = return_smoothing(hedge_fund_returns, ...
factor_returns, lag)
%
%#
%# # License
%#
%# =============================================================================
%#
%# Copyright 2011, Dimitrios Bisias, Andrew W. Lo, and Stavros Valavanis
%#
%# COPYRIGHT STATUS:  This work was funded in whole or in part by the Office of 
%# Financial Research under U.S. Government contract TOSOFR-11-C-0001, and is, 
%# therefore, subject to the following license: The Government is granted for 
%# itself and others acting on its behalf a paid-up, nonexclusive, irrevocable, 
%# worldwide license to reproduce, prepare derivative works,
%# distribute copies to the public, perform and display the work.  
%# All other rights are reserved by the copyright owner.
%# 
%# THIS SOFTWARE IS PROVIDED "AS IS".  YOU ARE USING THIS SOFTWARE AT YOUR OWN
%# RISK.  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
%# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
%# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS, CONTRIBUTORS, OR
%# THE UNITED STATES GOVERNMENT BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
%# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
%# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
%# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
%# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
%# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
%# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%# 
%# =============================================================================
%#
%# # Synopsis
%#
%# return_smoothing - Calculates the smoothing weights and the parameter ksi 
%# for a hedge fund
%#	
%# # Introduction
%#
%# It is a well documented empirical fact that returns to hedge funds and other
%# alternative investments are often highly serially correlated, and 
%# Getmansky, Lo, and Makarov (2004) explore several sources of such serial 
%# correlation and show that the most likely explanation is illiquidity exposure 
%# and smoothed returns. The authors propose an econometric model of return 
%# smoothing coefficients; they find that these vary considerably across hedge-
%# fund style categories and posit that they may be a useful proxy for 
%# quantifying illiquidity exposure. Their argument for why this is the case 
%# goes as follows: for illiquid securities, prices are not readily available 
%# and often, linear extrapolation methods of pricing are used, resulting in 
%# more persistent and smoother returns. Even when quotes are obtained from 
%# broker/dealers, the phenomenon still exists because broker/dealers also often 
%# use linear extrapolation pricing methods; furthermore, averaging quotes from 
%# multiple broker/dealers can lead to smoother profiles. Thus, hedge funds 
%# which invest in more illiquid securities/areas should exhibit a higher serial 
%# correlation in their reported returns. 
%#
%# The authors model this return smoothing and develop a measure to quantify the
%# effect. This is discussed in detail in Bisias et al. (2012), Section F.6, and 
%# specifically in equations (A.93) - (A.95). This code estimates smoothing 
%# weights and the parameter ksi.
%# 
%# # Input Description
%#
%# ## Input arg: hedge_fund_returns
%# The returns of the hedge fund.
%# ### Type: float
%# ### Range: (-inf,+inf)
%# ### Dimensions: Tx1 matrix
%#
%# 1.  Rows represent dates.
%#
%#
%# ## Input arg: factor_returns
%# The returns of a benchmark factor (like S&P 500).
%# ### Type: float
%# ### Range: (-inf,+inf)
%# ### Dimensions: Tx1 matrix
%#
%# 1.  Rows represent dates.
%#
%#
%# ## Input arg: lag
%# The number of past years incorporated into the smoothing process.
%# ### Type: float
%# ### Range: (0,+inf)
%# ### Dimensions: scalar
%#
%# # Output Description
%#
%# ## Output arg: thetas
%# The individual weightings assigned to each of the past years returns.
%# ### Type: float
%# ### Range: (-inf,+inf)
%# ### Dimensions: Nx1 matrix
%#
%# ## Output arg: ksi 
%# The sum of the squares of the smoothing weights or thetas.
%# ### Type: float
%# ### Range: (-inf,+inf)
%# ### Dimensions: scalar
%#
%# # Examples
%# NOTE: Numbers used in the examples are arbitrary valid values.  
%# They do not necessarily represent a realistic or plausible scenario.
%# 
%#     hedge_fund_returns = ...
%#     [ 0.2; -0.1; 0.1;-0.01; 0.15;-0.11;-0.09; 0.18];
%#     factor_returns = ...
%#     [ 0.10; 0.10;-0.09;-0.07;-0.08; 0.06;-0.21; 0.16]
%# 
%#     lag = 2;
%# 
%#     [thetas, ksi] = return_smoothing(hedge_fund_returns, ...
%#     factor_returns, lag)
%# 
%# # References
%#
%# [Getmansky, M., Lo, A. W., & Makarov, I. (2004). An econometric model of 
%# serial correlation and illiquidity in hedge fund returns. 
%# Journal of Financial Economics, 74(3), 529-609.] 
%# (http://papers.ssrn.com/sol3/papers.cfm?abstract_id=384700)
%#
%# [Bisias et al. (2012). A survey of systemic risk analytics 
%# (Working paper #0001). 
%# Washington, DC: Office of Financial Research, 129-133.]
%# (http://www.treasury.gov/initiatives/wsr/ofr/Documents/
%# OFRwp0001_BisiasFloodLoValavanis_ASurveyOfSystemicRiskAnalytics.pdf)
%#
%# # Code

% Run warning message
warning('OFRwp0001:UntestedCode', ...
    ['This version of the source code is very preliminary, ' ...
     'and has not been thoroughly tested. Users should not rely on ' ...
     'these calculations.']);




n = length(hedge_fund_returns);

% The first lag entries are ignored 
y = hedge_fund_returns(lag+1:end);
% Let's form the matrix X
X = ones(n-lag,lag+2);
for i=0:lag
    X(:,i+2)= factor_returns(lag+1-i:n-i);
end

gammas = regress(y,X);

thetas = gammas(2:end)/sum(gammas(2:end));

ksi = thetas'*thetas;


