function var = turbulence_var(asset_returns,portfolio, q)

%#
%# # License
%#
%# =============================================================================
%#
%# Copyright 2011, Dimitrios Bisias, Andrew W. Lo, and Stavros Valavanis
%#
%# COPYRIGHT STATUS:  This work was funded in whole or in part by the Office of 
%# Financial Research under U.S. Government contract TOSOFR-11-C-0001, and is, 
%# therefore, subject to the following license: The Government is granted for 
%# itself and others acting on its behalf a paid-up, nonexclusive, irrevocable, 
%# worldwide license to reproduce, prepare derivative works,
%# distribute copies to the public, perform and display the work.  
%# All other rights are reserved by the copyright owner.
%# 
%# THIS SOFTWARE IS PROVIDED "AS IS".  YOU ARE USING THIS SOFTWARE AT YOUR OWN
%# RISK.  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
%# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
%# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS, CONTRIBUTORS, OR
%# THE UNITED STATES GOVERNMENT BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
%# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
%# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
%# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
%# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
%# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
%# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%# 
%# =============================================================================
%#
%# # Synopsis
%#			
%# turbulence_var - calculates the value-at-risk of a portfolio using only data 
%# from turbulent periods. Turbulence is defined at a q-percentile
%# based on Kritzman et al. (2010)
%# 
%# # Introduction
%# NOTE: PART OF A SET OF 2 RELATED FILES:
%#
%#  * turbulence.m
%#  * turbulence_var.m
%#
%# Kritzman and Li (2010) define financial turbulence as a condition in which 
%# asset prices, given their historical patterns of behavior, behave in an 
%# uncharacteristic fashion, including extreme price moves, decoupling of 
%# correlated assets, and convergence of uncorrelated assets.  They quantify 
%# turbulence via the Mahalanobis distance (see Merton (1937)), which measures
%# the statistical unusualness of a set of returns given their historical 
%# pattern of behavior. Their measure is very general and can be applied across 
%# asset classes for which time-series return data are available.
%#
%# # Input Description
%#
%# ## Input arg: asset_returns
%# Time series returns for different asset classes.
%# ### Type: float
%# ### Range: (-inf,+inf)
%# ### Dimensions: TxK matrix
%#
%# 1.  Rows represent dates.
%# 2.  Columns represent returns for each of K asset classes.
%#
%#
%# ## Input arg: q 
%# The percentile at which we define the lower bound for turbulence.  
%# In the paper q = 0.75.
%# ### Type: float
%# ### Range: (0,1)
%# ### Dimensions: scalar
%#
%# ## Input arg: portfolio
%# The portfolio weights per asset class.
%# ### Type: float
%# ### Range: (0,1)
%# ### Dimensions: 1xK matrix
%#
%# 1.  Columns represent asset classes..
%#
%#
%# # Output Description
%#
%# ## Output arg: var
%# The value-at-risk of the portfolio consisting of only turbulent periods.  
%# Default used is 5% or .05.
%# ### Type: float
%# ### Range: (-inf,+inf)
%# ### Dimensions: scalar
%#
%# # Examples
%# NOTE: Numbers used in the examples are arbitrary valid values.  
%# They do not necessarily represent a realistic or plausible scenario.
%#
%#     asset_returns = ...
%#     [ 0.0595, 0.1211,-0.0806; 
%#      -0.1091, 0.0897,-0.0254; 
%#       0.0901, 0.0714,-0.0915; 
%#       0.1086, 0.0033,-0.1173;
%#       0.0614, 0.0151, 0.0291];
%#
%#     q = .75;
%#
%#     portfolio = [.33,.33,.33]
%#
%#     var = turbulence_var(asset_returns,portfolio, q)
%#
%# # References
%#
%# [Kritzman, M., & Li, Y. (2010). Skulls, financial turbulence, and 
%# risk management. Financial Analysts Journal, 66(5), 30-41.] 
%# (http://papers.ssrn.com/sol3/papers.cfm?abstract_id=1691756)
%#
%# [Bisias et al. (2012). A survey of systemic risk analytics 
%# (Working paper #0001). Washington, DC: Office of Financial Research, 80-81.]
%# (http://www.treasury.gov/initiatives/wsr/ofr/Documents/
%# OFRwp0001_BisiasFloodLoValavanis_ASurveyOfSystemicRiskAnalytics.pdf)
%#
%# # Code

% Run warning message
warning('OFRwp0001:UntestedCode', ...
    ['This version of the source code is very preliminary, ' ...
     'and has not been thoroughly tested. Users should not rely on ' ...
     'these calculations.']);


% Parameters:
% asset_returns A matrix of asset returns. Rows are different dates.
% Columns are different assets.
% portfolio The portfolio weights
% q The percentile
% Output:
% var the turbulence value-at-risk of the portfolio

[turbulence_series, threshold, turbulent_periods] = turbulence(asset_returns,q);

% Find the portfolio returns only at the turbulent periods
portfolio_returns = portfolio'*asset_returns(turbulent_periods,:);

% Find the variance-at-risk i.e. the 5% percentile
var = prctile(portfolio_returns,5);



