function undirected = undirected_banking_linkages(bank_claims, ...
 non_bank_claims, non_bank_liabilities)
%
%#
%# # License
%#
%# =============================================================================
%#
%# Copyright 2011, Dimitrios Bisias, Andrew W. Lo, and Stavros Valavanis
%#
%# COPYRIGHT STATUS:  This work was funded in whole or in part by the Office of 
%# Financial Research under U.S. Government contract TOSOFR-11-C-0001, and is, 
%# therefore, subject to the following license: The Government is granted for 
%# itself and others acting on its behalf a paid-up, nonexclusive, irrevocable, 
%# worldwide license to reproduce, prepare derivative works,
%# distribute copies to the public, perform and display the work.  
%# All other rights are reserved by the copyright owner.
%# 
%# THIS SOFTWARE IS PROVIDED "AS IS".  YOU ARE USING THIS SOFTWARE AT YOUR OWN
%# RISK.  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
%# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
%# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS, CONTRIBUTORS, OR
%# THE UNITED STATES GOVERNMENT BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
%# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
%# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
%# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
%# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
%# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
%# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%# 
%# =============================================================================
%#
%# # Synopsis
%#			
%# undirected_banking_linkages - Calculates the undirected network of banking 
%# linkages for n countries based on Fender and McGuire (2010).
%#
%# # Introduction
%#
%# Fender and McGuire (2010) emphasize the fact that large, international 
%# banks tend to have offices in many different countries, hence focusing on 
%# banking-group-level balance-sheet risks may ignore important exposures at 
%# the individual-office level since these risks are netted out at the group 
%# level. Such individual-office frictions may be important, especially during 
%# a crisis. A key implication of geographical diversity is that cross-border 
%# linkages of individual office locations can determine how shocks are 
%# transmitted from one location (country) to another. The authors propose 
%# creating undirected and directed networks at the local-office level to 
%# assess these risks.
%#
%# In creating an undirected network, the size of each node is proportional to 
%# the stock of cross-border claims and liabilities of reporting banks located 
%# in the particular geographical region. The thickness of a line between 
%# regions A and B is proportional to the sum of claims of banks in A on all 
%# residents of B, liabilities of banks in A to non-banks in B, claims of banks 
%# in B on all residents of A, and liabilities of banks in B to non-banks in A. 
%# Note that this undirected network is a static depiction of cross-border 
%# linkages.
%#
%# # Input Description
%#
%# ## Input arg: bank_claims
%# The (i,j) component shows the bank claims from country i to banks in 
%# country j.
%# ### Type: float
%# ### Range: [0,+inf)
%# ### Dimensions: NxN matrix (not necessarily symmetric)
%#
%# 1.  Rows represent each of N countries.
%# 2.  Columns represent each of N countries.
%# 
%#
%# ## Input arg: non_bank_claims
%# The (i,j) component shows the claims from banks in country i to non-banks in 
%# country j
%# ### Type: float
%# ### Range: [0,+inf)
%# ### Dimensions: NxN matrix (not necessarily symmetric).
%#
%# 1.  Rows represent each of N countries.
%# 2.  Columns represent each of N countries.
%#
%#
%# ## Input arg: non_bank_liabilities
%# The (i,j) component shows the liabilities of banks in i to non-banks in j.
%# ### Type: float
%# ### Range: [0,+inf)
%# ### Dimensions: NxN matrix (not necessarily symmetric).
%#
%# 1.  Rows represent each of N countries.
%# 2.  Columns represent each of N countries.
%#
%#
%# # Output Description
%# 
%# ## Output arg: undirected
%# A symmetric matrix with null diagonals depicting gross linkages (claims and
%# liabilities) between banks and non-banks in country pairs.
%# ### Type: float
%# ### Range: [0,+inf)
%# ### Dimensions: NxN matrix (symmetric).
%#
%# 1.  Rows represent banks in each of N countries.
%# 2.  Columns represent each of N countries.
%#
%#
%# # Examples
%# NOTE: Numbers used in the examples are arbitrary valid values.  
%# They do not necessarily represent a realistic or plausible scenario.
%#
%#     bank_claims=[100,101,102,103;
%#                  104,105,106,107;
%#                  108,109,110,111;
%#                  112,113,114,115];
%# 
%#     non_bank_claims=[116,117,118,119;
%#                      120,121,122,123;
%#                      124,125,126,127;
%#                      128,129,130,131];
%# 
%#     non_bank_liabilities=[132,133,134,135;
%#                           136,137,138,139;
%#                           140,141,142,143;
%#                           144,145,146,147];
%# 
%#     undirected = undirected_banking_linkages(...
%#               bank_claims, non_bank_claims, non_bank_liabilities);
%# 
%# # References
%#
%# [Fender, I., & McGuire, P. (2010). Bank structure, funding risk and the 
%# transmission of shocks across countries: concepts and measurement. 
%# BIS quarterly review, 2010, 63-79.] 
%# (http://www.bis.org/publ/qtrpdf/r_qt1009h.htm)
%#
%# [Bisias et al. (2012). A survey of systemic risk analytics 
%# (Working paper #0001). Washington, DC: Office of Financial Research, 69-73.]
%# (http://www.treasury.gov/initiatives/wsr/ofr/Documents/
%# OFRwp0001_BisiasFloodLoValavanis_ASurveyOfSystemicRiskAnalytics.pdf)
%#
%# # Code

% Run warning message
warning('OFRwp0001:UntestedCode', ...
    ['This version of the source code is very preliminary, ' ...
     'and has not been thoroughly tested. Users should not rely on ' ...
     'these calculations.']);


num_countries = size(bank_claims,1);

undirected = bank_claims + non_bank_claims + non_bank_liabilities;
% We need also to add the claims and liabilities from j to i making the
% matrix symmetric
undirected = undirected+undirected';

% Null the diagonal to avoid self-loops
for i=1:num_countries
    undirected(i,i)=0;
end




